from dataclasses import dataclass
from typing import Union

@dataclass
class Quote:
	symbol: str
	description: str
	digits: int
	ask: Union[float, int]
	bid: Union[float, int]
	change: Union[float, int]
	minmax: dict
	change24h: Union[float, int]
	position: dict
	timestamp: Union[float, int]
	change24h_percent: Union[float, int]
	rate_open: dict
	rate_24h: dict
	rate_7d: dict
	rate_30d: dict
	minmax_52w: dict
	minmax_7d: dict
	about: Union[None, str]

@dataclass
class Tick:
	open: Union[float, int]
	high: Union[float, int]
	close: Union[float, int]
	low: Union[float, int]
	date: int

@dataclass
class SubTick:
	digits: Union[float, int]
	ask: Union[float, int]
	bid: Union[float, int]
	change: Union[float, int]
	symbol: str
	lasttime: Union[float, int]

@dataclass
class Signal:
	comment: str
	pair: str
	cmd: str
	trading_system: int
	price: Union[float, int]
	sl: Union[float, int]
	tp: Union[float, int]
	period: str
	author: bool
	timestamp: int
	cmd_id: int
	trading_system_id: int
	pair_description: str


import requests
import json
from websocket import create_connection
from .errors import *
from .schemas import *

Q_URL = 'https://ark.fx.co/get-quotes-info'
HLOC_URL = 'https://ark.fx.co/get-quotes-hloc'
SIG_URL = 'https://ark.fx.co/get-signals'
ACCESS_TOKEN = 'eg4q3i2697y9ws42b10p469s'
HEADERS = {
	'Authorization': 'Bearer ' + ACCESS_TOKEN,
	'User-Agent': 'RestSharp/107.3.0.0'
}

class ForexPortalAPI():
	def __init__(self):
		pass

	def getQuoteInfo(self, quote):
		resp = requests.get(
			Q_URL,
			headers = HEADERS,
			params = {
				'_format': 'json',
				'pair': quote
			}
		).json()

		if resp['status'] != 'ok':
			msg = resp['message'] if 'message' in 'resp' \
				else resp['description']
			error = f'Unable to fetch quote "{quote}". Error: {msg}'
			raise UnableToFetchError(error)

		return Quote(**resp['result'][list(resp['result'].keys())[0]])

	def getQuoteHLOC(self, quote, period = 'H1', limit = 400):
		resp = requests.get(
			HLOC_URL,
			headers = HEADERS,
			params = {
				'_format': 'json',
				'pair': quote,
				'limit': limit,
				'period': period
			}
		).json()

		if resp['status'] != 'ok':
			msg = resp['message'] if 'message' in 'resp' \
				else resp['description']
			error = f'Unable to fetch quote "{quote}". Error: {msg}'
			raise UnableToFetchError(error)

		return [
			Tick(
				open = a['Open'],
				high = a['High'],
				low = a['Low'],
				close = a['Close'],
				date = a['Date']
			) for a in resp['result'][list(resp['result'].keys())[0]]['ticks']
		]

	def subscribeQuote(self, quote):
		ws = create_connection('wss://api2-webtrader.ifxdb.com/ws/?EIO=3&token=public&transport=websocket')
		ws.recv()
		ws.recv()
		ws.send('42["tick:subscribe",{"symbols":["' + quote + '"],"type":"standard","digits":4}]')

		while True:
			obj = json.loads(ws.recv()[2:])

			if obj[0] != 'event:quote:tick:standard:4':
				continue

			yield SubTick(**obj[1]['msg'])

	def getQuoteSignals(self, quote, limit = 50):
		resp = requests.get(
			SIG_URL,
			headers = HEADERS,
			params = {
				'from': 0,
				'_format': 'json',
				'pair': quote,
				'limit': limit,
				'trading_system': 3
			}
		).json()
		
		if resp['status'] != 'ok':
			msg = resp['message'] if 'message' in 'resp' \
				else resp['description']
			error = f'Unable to fetch quote "{quote}". Error: {msg}'
			raise UnableToFetchError(error)
		if resp['result'][list(resp['result'].keys())[0]]['pair'] != quote.upper():
			error = f'Unable to fetch quote "{quote}". Error: Invalid quote'
			raise UnableToFetchError(error)

		return [
			Signal(**resp['result'][a]) for a in resp['result']
		]


from setuptools import setup

with open("README.md", 'r', encoding = 'utf-8') as file:
    read_me_description = file.read()

setup(name='forexportal-api',
      version='0.2',
      description='Forex Portal API',
      packages=['forexportal'],
      author_email='me@biteof.space',
      long_description=read_me_description,
      long_description_content_type="text/markdown",
      url="https://github.com/btfspace/forexportal-api",
      zip_safe=False,
      install_requires=['requests', 'websocket', 'websocket-client'],
      classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
      ],
)